/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

/*
 * GDB Source/Sink which is designed to be used in conjunction with the python
 * gdbDriver in the AudioIO project.
 */
#include <malloc.h>
#include <string.h>

#include "ioGDB.h"

OperationParameters gdbParameters;

uint32_t gdbInputSize;
unsigned char gdbInput[CODEC_INPUT_SIZE];

uint32_t gdbOutputSize;
unsigned char gdbOutput[CODEC_OUTPUT_SIZE];

/* Define function prototypes for the features we will override from the base */
static void gdbRead(pSourceSink source, unsigned char *buffer, uint32_t maxsize);

static void gdbWrite(pSourceSink sink, unsigned char *buffer, uint32_t size);

/* ----------------------------------------------------------------------------
 * Function     : makeGDBSourceSink
 * ----------------------------------------------------------------------------
 * Description  : This IO provider allows test data to be input and output
 *                to the codec and is provided for debug purposes. This
 *                method allows the construction of a suitable control block
 *                using malloc.
 * Inputs       : None
 * Outputs      : Pointer to an IO Source/Sink control block which has been
 *                configured to match the test data indicated above.
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
pSourceSink makeGDBSourceSink(void)
{
    pSourceSink object = (pSourceSink)malloc(sizeof(struct _SourceSink));
    if (object != NULL)
    {
        object->read = &gdbRead;
        object->write = &gdbWrite;
    }

    return object;
}

/* ----------------------------------------------------------------------------
 * Function     : gdbReadComplete
 * ----------------------------------------------------------------------------
 * Description  : Marker function which is used by the GDB debugger to
 *                identify when data may be injected or extracted from the
 *                running system
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
void gdbReadComplete(void)
{
    /* don't need anything in here, it is a hook for the debugger */
}

/* ----------------------------------------------------------------------------
 * Function     : gdbRead
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the read method to provide data for test
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which will be used to return the test
 *                data
 *                maxsize : The maximum size of the buffer which must not be
 *                exceeded
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
void gdbRead(pSourceSink source, unsigned char *buffer, uint32_t maxsize)
{
    if (gdbInputSize > maxsize)
    {
        gdbInputSize = maxsize;
    }
    memcpy(buffer, gdbInput, gdbInputSize);
    memcpy(&source->operation, &gdbParameters, sizeof(OperationParameters));
    source->srcUsed = gdbInputSize;
    gdbReadComplete();
}

/* ----------------------------------------------------------------------------
 * Function     : gdbWriteComplete
 * ----------------------------------------------------------------------------
 * Description  : Marker function which is used by the GDB debugger to
 *                identify when data may be injected or extracted from the
 *                running system
 * Inputs       : None
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
void gdbWriteComplete(void)
{
    /* don't need anything in here, it is a hook for the debugger */
}

/* ----------------------------------------------------------------------------
 * Function     : gdbWrite
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the write method to verify test results
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which has been used to provide the
 *                test results
 *                size : The used size of the buffer
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
void gdbWrite(pSourceSink sink, unsigned char *buffer, uint32_t size)
{
    gdbOutputSize = (size > CODEC_OUTPUT_SIZE) ? CODEC_OUTPUT_SIZE : size;
    memcpy(gdbOutput, buffer, gdbOutputSize);
    sink->snkUsed = gdbOutputSize;
    gdbWriteComplete();
}
