/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#ifndef _CODEC_WRAPPER_H_
#define _CODEC_WRAPPER_H_

#include <rsl10_lpdsp32.h>

#include "codecs/baseDSP/baseDSPCodecInternal.h"

extern "C"
{

/* ----------------------------------------------------------------------------
 * Function     : configure
 * ----------------------------------------------------------------------------
 * Description  : Method to provide a means to configure a codec for first 
 *                use. 
 * Inputs       : pConfig : Pointer to a codec configuration block which has 
 *                been set up with any information required to allow the 
 *                configuration. For instance sample rates and frame sizes 
 *                should be pre-initialised. Exactly what information needs 
 *                to be set up is codec dependent 
 * Outputs      : No outputs but the configuration process will potentially 
 *                change the internal state of the codec. 
 * Assumptions  : The configuration block must be pre-initialised. 
 * ------------------------------------------------------------------------- */
    void configure(pBaseDSPCodec pConfig);

/* ----------------------------------------------------------------------------
 * Function     : encodeReset
 * ----------------------------------------------------------------------------
 * Description  : Method to provide a mechanism to reset the encode portion 
 *                of a codec to its initial state. 
 * Inputs       : pConfig : Pointer to a codec configuration block 
 * Outputs      : No outputs but the configuration process will potentially 
 *                change the internal state of the codec. 
 * Assumptions  : The configuration block must be pre-initialised. 
 * ------------------------------------------------------------------------- */
    void encodeReset(pBaseDSPCodec pConfig);

/* ----------------------------------------------------------------------------
 * Function     : decodeReset
 * ----------------------------------------------------------------------------
 * Description  : Method to provide a mechanism to reset the decode portion 
 *                of a codec to its initial state. 
 * Inputs       : pConfig : Pointer to a codec configuration block 
 * Outputs      : No outputs but the configuration process will potentially 
 *                change the internal state of the codec. 
 * Assumptions  : The configuration block must be pre-initialised. 
 * ------------------------------------------------------------------------- */
    void decodeReset(pBaseDSPCodec pConfig);

/* ----------------------------------------------------------------------------
 * Function     : encode
 * ----------------------------------------------------------------------------
 * Description  : Method to encode a block of samples using the specified 
 *                codec. 
 * Inputs       : pConfig : Pointer to a codec configuration block which 
 *                specifies the control parameters for the encode operation 
 *                inBuffer : a pointer to a buffer containing the samples to 
 *                be encoded 
 * Outputs      : outBuffer : a pointer to a buffer that can be used to 
 *                write the encoded information 
 * Assumptions  : The configuration block has been pre-initialised and the 
 *                input and output buffers are sized appropriately for the 
 *                requested operation 
 * ------------------------------------------------------------------------- */
    void encode(pBaseDSPCodec pConfig, unsigned char *inBuffer, unsigned char *outBuffer);

/* ----------------------------------------------------------------------------
 * Function     : decode
 * ----------------------------------------------------------------------------
 * Description  : Method to decode a block of samples using the specified 
 *                codec. 
 * Inputs       : pConfig : Pointer to a codec configuration block which 
 *                specifies the control parameters for the decode operation 
 *                inBuffer : a pointer to a buffer containing the samples to 
 *                be decoded 
 *                usePLC : flag indicating that the packet is missing and the
 *                PLC algorithm should be used to conceal it
 * Outputs      : outBuffer : a pointer to a buffer that can be used to 
 *                write the decoded samples 
 * Assumptions  : The configuration block has been pre-initialised and the 
 *                input and output buffers are sized appropriately for the 
 *                requested operation 
 * ------------------------------------------------------------------------- */
    void decode(pBaseDSPCodec pConfig, unsigned char *inBuffer, unsigned char *outBuffer, bool usePLC);

}

#endif
