/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * rsl10_sys_crc.2h
 * - Cyclic-Redundancy Check (CRC) peripheral support
 * ----------------------------------------------------------------------------
 * $Revision: 1.7 $
 * $Date: 2017/10/19 14:28:12 $
 * ------------------------------------------------------------------------- */

#include <string.h>
#include "mem_func.h"

/* Define a heap that is 8-byte aligned to be used for allocating memory */
static unsigned int chess_storage(DMA%8) memVar[MALLOC_HEAP_SIZE >> 2];

/* Initialise these to be dumb so that we can't start allocating memory unless
 * we have initialised the allocator
 */
static unsigned int alloc_start_addr    = 0xFFFFFFFF;
static unsigned int alloc_end_addr      = 0xFFFFFFFF;
static unsigned int alloc_total_size    = 0;
static unsigned int alloc_current_size  = 0xFFFFFFFF;

typedef struct memLink 
{
    unsigned int        size;
    struct memLink      *next;
    struct memLink      *prev;
} memoryLink;

/* ----------------------------------------------------------------------------
 * Function      : int alloc_init(unsigned int size, int clear)
 * ----------------------------------------------------------------------------
 * Description   : Initialize the allocation variables.
 * Inputs        : - size   - Number of words that can be allocated 
 *                 - clear  - If true, memory is initialized to zero
 * Outputs       : return value  - Size of memory in bytes that can be 
 *                                 allocated; -1 if a failure occured
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
int alloc_init(unsigned int size, int clear)
{
    memoryLink *blk;
    unsigned int start = (unsigned int) &memVar[0];

    if (start & 0x07)
    {
        return(-1);
    }

    if (size & 0x07)
    {
        return(-1);
    }

    /* Convert to reflect number of bytes */
    size = size * sizeof(unsigned int);
    if (size > MALLOC_HEAP_SIZE)
    {
        return(-1);
    }

    alloc_start_addr = start;
    alloc_end_addr = start + size;
    alloc_total_size = size;
    alloc_current_size = sizeof(memoryLink);

    if (clear) 
    {
        memset((void *)(alloc_start_addr), 0, size);
    }

    blk = (memoryLink *)alloc_start_addr;
    blk->size = sizeof(memoryLink);
    blk->prev = (memoryLink *)0;
    blk->next = (memoryLink *)0;

    return size;
}

/* ----------------------------------------------------------------------------
 * Function      : void * malloc_ (unsigned int size)
 * ----------------------------------------------------------------------------
 * Description   : Memory allocation
 * Inputs        : - size   - Number of bytes to allocate
 * Outputs       : return value  - Pointer to the allocated memory
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void * malloc_(unsigned int size)
{
    memoryLink *blk = (memoryLink *)alloc_start_addr;
    memoryLink *new_blk;

    size += sizeof(memoryLink);
    size = (size + 7) & ~0x7;
    if ((alloc_current_size + size) > alloc_total_size) 
    {
        return (NULL);
    }

    do 
    {
        new_blk = (memoryLink *)((unsigned long)blk + blk->size);

        if ( blk->next ) 
        {
            if ( ( (unsigned long)(blk->next) - (unsigned long)new_blk ) >= size ) break;
            blk = blk->next;
        } 
        else 
        {
            if ( ( alloc_end_addr - (unsigned long)new_blk ) >= size ) break;
            else 
            {
                new_blk = (memoryLink *)0;
            }
        }
    } 
    while ( new_blk );

    if ( new_blk ) 
    {
        new_blk->size = size;
        new_blk->next =blk->next;
        new_blk->prev = blk;
        blk->next = new_blk;
        new_blk = (memoryLink *)((unsigned long)new_blk + sizeof(memoryLink));
        alloc_current_size += size;
    }
    
    return( (void *)new_blk );
}

/* ----------------------------------------------------------------------------
 * Function      : void free_(void *m)
 * ----------------------------------------------------------------------------
 * Description   : Release allocated memory
 * Inputs        : - m  - Allocated memory to release
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void free_(void *m)
{
    memoryLink *blk = (memoryLink *)((unsigned long)m - sizeof(memoryLink));

    if ( blk->prev ) blk->prev->next = blk->next;
    if ( blk->next ) blk->next->prev = blk->prev;

    alloc_current_size -= blk->size;

    blk->prev = 0;
    blk->next = 0;
    blk->size = 0;
}

/* ----------------------------------------------------------------------------
 * Function      : void *calloc_(unsigned int num, unsigned int siz)
 * ----------------------------------------------------------------------------
 * Description   : local implementation of "calloc"
 * Inputs        : num - The number of elements to allocate
 * 				   siz - the size of each element being allocated
 * Outputs       : pointer to the allocated buffer or NULL if insufficient
 *                 memory available
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
void *calloc_(unsigned int num, unsigned int siz)
{
	void *new_blk = malloc_(num * siz);
	if (new_blk)
		memset(new_blk, 0, num * siz);
	return new_blk;
}


